/*
   DScope Application

   Copyright (C) 2008,2009:
         Daniel Roggen, droggen@gmail.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include <QKeyEvent>
#include <QMessageBox>
#include <QPainter>
#include <QDebug>
#include <QtNetwork> 
#include <QTcpSocket>
#include <QtGui/QStatusBar>
#include <QXmlStreamWriter>
#include <QFileDialog>
#include <vector>
#include <fstream>
#include <sstream>
#include <iostream>
#include <iterator>
#include "MainWindow.h"
#include "portwindowimpl.h"
#include "helpwindowimpl.h"

#include "DScopeQT.h"


#include "cio.h"
#include "precisetimer.h"

#include "FrameParser/FrameParser.h"
#include "FrameParser/FrameParser2.h"
#include <QExtSerialPort/qextserialenumerator.h>
#include <QExtSerialPort/qextserialport.h>

using namespace std;

/*
 Example format:
 <Calibrated;5 6 7;xscale=1;yscale=-3300 3300 ;color=ff0000 ff00 ff><Raw;2 3 4;xscale=1;yscale=0 1024 ;color=ff0000 ff00 ff>
*/


//-------------------------------------------------------------------------------------------------
// SplitInVector
//-------------------------------------------------------------------------------------------------
// Returns false in case of failure.
// Only one split character is allowed
template<class T> bool SplitInVector(string str,string split,vector<T> &r,ios_base&(&b)(ios_base&)=dec)
{
	
	r.clear();
	QString qstr(str.c_str());
	QStringList sl = qstr.split(QString(split.c_str()),QString::SkipEmptyParts);
	for(int i=0;i<sl.size();i++)
	{
		std::istringstream iss(sl[i].toStdString());
		T j;
		if( !((iss >> b >> j).fail()))
			r.push_back(j);
		else
			return false;
	}
	return true;	
}
//-------------------------------------------------------------------------------------------------
// ParseLayoutString
// Layout string: n_scopes<<title>;<traces>[;xscale=<xs>][;yscale=<auto|min;max>][color=<colors>]>
//-------------------------------------------------------------------------------------------------
// Example: <Scope 2;4;yscale=-100 100>
int ParseLayoutString(string str,ScopesDefinition &scopesdefs)
{
	scopesdefs.scopedefinition.clear();
	
	// Regexp for layout string:
	// Characters ;<> are excluded in the match of the expression for syntax checking.
	// Group 0: whole expression
	// Group 1: title
	// Group 2: traces
	// Group 4: xscale
	// Group 6: yscale
	// Group 8: color
	// Validation of the whole string: look for multiple expressions with a '(expression)*' regex string.
	// Original string:
	//string expression = string("\\s*<\\s*([^;^>^<]*?)\\s*;\\s*([^;^>^<]+?)\\s*(;\\s*xscale\\s*=\\s*([^;^>^<]*?))?(;\\s*yscale\\s*=\\s*([^;^>^<]*?))?(;\\s*color\\s*=\\s*([^;^>^<]*?))?>\\s*");
	
	// Beware QRegExp does not support non greedy match *? and +? (anyway isn't needed)
	string expression = string("\\s*<\\s*([^;^>^<]*)\\s*;\\s*([^;^>^<]+)\\s*(;\\s*xscale\\s*=\\s*([^;^>^<]*))?(;\\s*yscale\\s*=\\s*([^;^>^<]*))?(;\\s*color\\s*=\\s*([^;^>^<]*))?>\\s*");
	QRegExp e(QString(expression.c_str()));
	//printf("e.isValid: %d\n",(int)e.isValid());
	//printf("e.isValid: %s\n",e.errorString().toStdString().c_str());
	string expression2=string("(")+expression+string(")*"); 
	QRegExp validation(QString(expression2.c_str()));
	//printf("validation.isValid: %d\n",(int)validation.isValid());
	//printf("validation.isValid: %s\n",e.errorString().toStdString().c_str());


	// Validation	
	int iin=validation.exactMatch(QString(str.c_str()));
	if(iin==-1)
	{
		printf("Validation match failed\n");
		return -5;
	}
		

	// Expression parsing
	e.indexIn(QString(str.c_str()));
	unsigned midx=0;
	int pos = 0;
	while((pos = e.indexIn(QString(str.c_str()), pos)) != -1)
	{
		pos += e.matchedLength();
	
		/*printf("Capture group %d\n",midx);	
		printf("NumCaptures: %d\n",e.numCaptures());
		for(int i=0;i<e.numCaptures()+1;i++)
			printf("Capture %d: %s\n",i,e.cap(i).toStdString().c_str());*/
			
			
		ScopeDefinition sd;
		if(e.numCaptures()!=8)
		{
			cout << "Invalid match size\n";
			return -9;
		}
		
		sd.title = e.cap(1).toStdString();
		string traces = e.cap(2).toStdString();
		string xscale = e.cap(4).toStdString();
		string yscale = e.cap(6).toStdString();
		string color = e.cap(8).toStdString();
		
		vector<unsigned> d;
		vector<int> dxscale,dyscale;
		vector<string> dstring;
		
		if(!SplitInVector(traces,string(" "),sd.traces))
			return -20;
		cout << "Traces: ";
		for(unsigned j=0;j<sd.traces.size();j++)
			cout << sd.traces[j] << "; ";
		cout << "\n";
		
		// Parse the X scale.
		if(!SplitInVector(xscale,string(" "),dxscale))
			return -21;
		if(dxscale.size()>1)
			return -22;
		if(dxscale.size()==0)
			sd.xscale=1;
		else
			sd.xscale=dxscale[0];
		cout << "xscale: " << sd.xscale << "\n";
		

		// Parse the Y scale.
		if(!SplitInVector(yscale,string(" "),dstring))
			return -23;
		if(dstring.size()>2)
			return -24;
		switch(dstring.size())
		{
			case 0:
				sd.yauto=true;
				break;
			case 1:
				if(dstring[0].compare("auto")==0)
					sd.yauto=true;
				else
					return -25;
				break;
			case 2:
				sd.yauto=false;
				if(!SplitInVector(yscale,string(" "),sd.yscale))
					return -26;
				if(sd.yscale.size()!=2)
					return -27;
				if(sd.yscale[0]>sd.yscale[1])
				{
					int t=sd.yscale[0];
					sd.yscale[0]=sd.yscale[1];
					sd.yscale[1]=t;
				}
				if(sd.yscale[0]==sd.yscale[1])
					sd.yscale[0]--;
				break;
		}
		cout << "yscale: ";
		if(sd.yauto==true)
			cout << "auto\n";
		else
			cout << "[" << sd.yscale[0] << ";" << sd.yscale[1] << "]\n";
		
		// Parse the colors
		if(!SplitInVector(color,string(" "),sd.colors,hex))
			return -28;
		cout << "color: ";
		for(unsigned j=0;j<sd.colors.size();j++)
			cout << sd.colors[j] << "; ";
		cout << "\n";
		
		if(sd.colors.size()!=0 && (sd.colors.size() != sd.traces.size()))
			return -29;

		cout << "\n";
		scopesdefs.scopedefinition.push_back(sd);

		midx++;
	}
	
	
	
	return 0;
}


//-------------------------------------------------------------------------------------------------
// MainWindowementation
//-------------------------------------------------------------------------------------------------



//
MainWindow::MainWindow( QWidget * parent, Qt::WFlags f)
	: QMainWindow(parent, f),
	fp(string(""))
{
	//ConsoleInit();
	

	
	setupUi(this);
	
	

	
	connect(actionConnect, SIGNAL(triggered()), this, SLOT(connectClicked()));
	connect(uiRefreshRate,SIGNAL(valueChanged(int)),this,SLOT(SetRefreshRate(int)));
	connect(uiAfterGlow,SIGNAL(stateChanged(int)),this,SLOT(SetAfterGlow(int)));
	
	
	connect(action_Save_configuration, SIGNAL(triggered()), this, SLOT(saveConfiguration()));
	connect(action_Load_configuration, SIGNAL(triggered()), this, SLOT(loadConfigurationAction()));
	connect(action_Exit, SIGNAL(triggered()), this, SLOT(actionExit()));
	connect(action_About, SIGNAL(triggered()), this, SLOT(actionAbout()));
	connect(action_List_serial_ports, SIGNAL(triggered()), this, SLOT(actionListPorts()));
	connect(action_How_to, SIGNAL(triggered()), this, SLOT(actionHowto()));

	
	connect(uipbConnect, SIGNAL(clicked()), this, SLOT(connectClicked()));
	connect(uipbApplyDisplay, SIGNAL(clicked()), this, SLOT(applyClicked()));
	connect(pushButton, SIGNAL(clicked()), this, SLOT(testClicked()));
	connect(uipbApplyBinary, SIGNAL(clicked()), this, SLOT(applyBinary()));
	

	// Initialize the scopes
	dscopes = new DScopesQTWidget(0,0,640,480,false);
	mainVerticalLayout->addWidget(dscopes);
	resize(800, 640);	
	
	// Initialize the afterglow to reflect the scope
	uiAfterGlow->setCheckState(Qt::Unchecked);
	
	// Initialize the refreshrate
	uiRefreshRate->setValue(10);
	SetRefreshRate(10);
	

	
	tcpSocket = new QTcpSocket(this);
	port=0;
	binary=false;		// Text mode by default
	
	printf("Read buffer size: %d\n",tcpSocket->readBufferSize());

	connect(tcpSocket, SIGNAL(readyRead()), this, SLOT(tcpRead()));
	connect(tcpSocket, SIGNAL(error(QAbstractSocket::SocketError)),this, SLOT(displayError(QAbstractSocket::SocketError)));


	uileHostPort->setText(QString("ser:com4:19200"));

	pt=0;
	lastplot=0;
	

	fp.Status();
	
	Connected=0;
	

	time_received_last=0;
	time_received_delta=0;
	time_displayed_last=PreciseTimer::QueryTimer();
	time_displayed_delta=0;
	
	datarateLabel=new QLabel(statusBar());
	statusBar()->addWidget(datarateLabel);
	displayrateLabel=new QLabel(statusBar());
	statusBar()->addWidget(displayrateLabel);
	
	time_received_delta_vector.resize(100,0.0);
	
	QStringList arg = qApp->arguments();
	if(arg.contains("-f"))
	{
		int idx=arg.indexOf("-f");
		if(idx<arg.size()-1)
			loadConfiguration(arg.at(idx+1));
		else
			QMessageBox::critical(this, "Command line error", "Missing configuration file name to load");
	}
	if(arg.contains("-c"))
	{
		connectClicked();
	}
}



void MainWindow::Plot()
{
	double t1,t2;
	//t1=PreciseTimer::QueryTimer();
	// Plot scopes
	if(sd.scopedefinition.size()==0)
	{
		for(int i=0;i<alldata.size();i++)
			dscopes->operator[](handles[i])->Plot(alldata[i],0xffffffff);
	}
	else
	{
		// Iterate all scopes
		for(unsigned s=0;s<sd.scopedefinition.size();s++)
		{
			// Iterate all traces of the scope
			vector<vector<int> *> vd;		// data for the traces in this scope
			vector<unsigned> vc;			// colors  of the traces
			for(unsigned t=0;t<sd.scopedefinition[s].traces.size();t++)
			{
				// If trace available
				if(sd.scopedefinition[s].traces[t]<alldata.size())
				{
					vd.push_back(&alldata[sd.scopedefinition[s].traces[t]]);
					if(sd.scopedefinition[s].colors.size()!=0)
						vc.push_back(sd.scopedefinition[s].colors[t]);
					else
						vc.push_back(0xffffffff);
				}
				dscopes->operator[](handles[s])->Plot(vd,vc);
			}
		}
	}
	/*t2=PreciseTimer::QueryTimer();
	pt = 0.9*pt+0.1*(t2-t1);
	printf("plot time: %.5lf\n",pt);*/
//	dscopes->Render();
}
void MainWindow::timerEvent(QTimerEvent *event)
{
	double f=0.9;
	double t = PreciseTimer::QueryTimer();
	time_displayed_delta = f*time_displayed_delta + (1.0-f)*(t-time_displayed_last);
	time_displayed_last=t;
	QString s;
	s.sprintf("Display rate: %3.0lf Hz",1.0/time_displayed_delta);
	displayrateLabel->setText(s);

	Plot();
	dscopes->Render();
	dscopes->repaint();

}



void MainWindow::connectClicked()
{
	if(Connected!=0)
	{
		// If connected, disconnect.
		if(Connected==1)
		{
			// TCP disconnect
			if( tcpSocket->state() != QAbstractSocket::UnconnectedState )
			{
				tcpSocket->disconnectFromHost();
				if(tcpSocket->state() != QAbstractSocket::UnconnectedState)
				{
					if(!tcpSocket->waitForDisconnected())
					{
						QMessageBox::critical(this, "Disconnect failed", "Could not disconnect from host\n");
						return;
					}
				}
			}
		}
		else
		{
			// Serial disconnect
			printf("Before close\n");
			port->close();
			printf("After close\n");
			//if(port) { delete port; port=0; }
		}
		Connected=0;
		// Change the button text.
		uipbConnect->setText("&Connect");
	}
	else
	{
		// If not connected, connect.	
		// tcp:host:port
		// ser:com:speed
		// Regexp: spaces (tcp or ser) spaces colon spaces (host/com) spaces colon spaces (port/speed) spaces
		// Capture 0: entire string. Capture 1: tcp|ser. Capture 2: host|com. Capture 3: port|speed
		QRegExp v("\\s*(tcp|ser)\\s*:\\s*(.+)\\s*:\\s*(\\d+)\\s*");
		bool expok=true;
		bool tcp;
		int index=v.indexIn(uileHostPort->text().toLower());
		if(v.cap(1).compare("tcp")==0)
		{
			tcp=true;
		}
		else
		{
			if(v.cap(1).compare("ser")==0)
				tcp=false;
			else
				expok=false;
		}
	
		bool cap3ok;		 
		unsigned cap3 = v.cap(3).toUInt(&cap3ok);
		if(index==-1 || cap3ok==false || expok==false)
		{
			QMessageBox::critical(this, "Invalid device specification", "Device is specified as follows:\nser:com:speed\ntcp:host:port\nExample: \"127.0.0.1:50008\"");
			return;
		}
		printf("Connect as: %d %s %d\n",tcp,v.cap(2).toStdString().c_str(),cap3);
		
		if(tcp)
		{
			// Connect to host	
			tcpSocket->connectToHost(v.cap(2),cap3);
			if (!tcpSocket->waitForConnected(2000))
			{
				QMessageBox::critical(this, "Connection failed", "Connection to host failed. Check host:port settings.\n");
				return;
			}
			Connected=1;
		}
		else
		{
			if(cap3!=110&& cap3!=300&& cap3!=600&& cap3!=1200&& cap3!=2400&& cap3!=4800&& cap3!=9600&& cap3!=14400&&
			 cap3!=19200&& cap3!=38400&& cap3!=57600&&  cap3!=115200) 
			{
			 	QMessageBox::critical(this, "Invalid parameter", "Invalid baud rate.");
				return;
		 	}
			if(port) { delete port; port=0; }
			printf("port to open: %s\n",v.cap(2).toStdString().c_str());
			// This will be windows only:
			string portstring=string("\\\\.\\\\") + v.cap(2).toStdString();
			printf("win port to open: %s\n",portstring.c_str());
			port = new QextSerialPort(portstring.c_str(),QextSerialBase::EventDriven);
			//port = new QextSerialPort(v.cap(2).toStdString().c_str(),QextSerialBase::EventDriven);
			port->open(QIODevice::ReadOnly);
			if(port->isOpen())
			{
    			switch(cap3)
    			{
    				case 110:
    					port->setBaudRate(BAUD110);
    					break;
    				case 300:
    					port->setBaudRate(BAUD300);
    					break;
    				case 600:
    					port->setBaudRate(BAUD600);
    					break;
    				case 1200:
    					port->setBaudRate(BAUD1200);
    					break;
    				case 2400:
    					port->setBaudRate(BAUD2400);
    					break;
    				case 4800:
    					port->setBaudRate(BAUD4800);
    					break;
    				case 9600:
    					port->setBaudRate(BAUD9600);
    					break;
    				case 14400:
    					port->setBaudRate(BAUD14400);
    					break;
    				case 19200:
    					port->setBaudRate(BAUD19200);
    					break;
    				case 38400:
    					port->setBaudRate(BAUD38400);
    					break;
    				case 57600:
    					port->setBaudRate(BAUD57600);
    					break;
    				default:
    					port->setBaudRate(BAUD115200);
   				}
				port->setDataBits(DATA_8);
				port->setFlowControl(FLOW_OFF);
				port->setParity(PAR_NONE);
				port->setStopBits(STOP_1);
				//ps.Timeout_Millisec=500;
				port->setTimeout(500);
				connect(port,SIGNAL(readyRead()),this,SLOT(serialRead()));			
				connect(port,SIGNAL(aboutToClose()),this,SLOT(aboutToClose()));
				connect(port,SIGNAL(dsrChanged(bool)),this,SLOT(dsrChanged(bool)));
				Connected=2;
			}
			else
			{
				QMessageBox::critical(this, "Connection failed", "Cannot open serial device. Check port:speed settings.\n");
				return;
			}
		}
		time_received_last=PreciseTimer::QueryTimer();
		uipbConnect->setText("Dis&connect");
		

	}
}
void MainWindow::aboutToClose()
{
        printf("MainWindow::aboutToClose\n");
}


void MainWindow::receivedData(vector<int>&linedata)
{
	// Keep a buffer of the last 
	
	double f=0.99;
	double t = PreciseTimer::QueryTimer();
	if(linedata.size())
	{
	
		time_received_delta_vector.push_back(t-time_received_last);
		time_received_delta = f*time_received_delta + (1.0-f)*(t-time_received_last);

		time_received_delta_vector.erase(time_received_delta_vector.begin());
		double tt=0;
		for(int i=0;i<time_received_delta_vector.size();i++)
			tt+=time_received_delta_vector[i];
		tt/=(double)time_received_delta_vector.size();
			
		QString s;
		//s.sprintf("Data rate: %3.1lf Hz  %3.1lf Hz",1.0/time_received_delta,1.0/tt);
		s.sprintf("Data rate: %3.0lf Hz",1.0/tt);
		//s.sprintf("Data rate: %3.0lf Hz",1.0/time_received_delta);
		datarateLabel->setText(s);


		time_received_last=t;


	}
	
	
	

	
	const unsigned bufferlength = 10000;
	unsigned newsize = linedata.size();
	unsigned oldsize = alldata.size();
	// Check for sensor changes
	if(oldsize!=newsize)
	{
		printf("Changing from %d to %d\n",oldsize,newsize);
		
		// Resize the data vector according to the new number of sensors
		// Since we cannot identify sensors we should in principle erase all past data to avoid confusion - we don't.
		if(newsize<oldsize)
		{
			for(unsigned i=0;i<oldsize-newsize;i++)
				alldata.erase(alldata.end()-1);
		}
		else
		{
			for(unsigned i=0;i<newsize-oldsize;i++)
				alldata.push_back(vector<int> (bufferlength,0));
		}
		// Adjust scope number
		if(sd.scopedefinition.size()==0)
		{
			if(newsize>oldsize)
			{
				for(unsigned i=0;i<newsize-oldsize;i++)
					handles.push_back(dscopes->AddScope());
			}
			else
			{
				for(unsigned i=0;i<oldsize-newsize;i++)
				{
					dscopes->RemoveScope(*(handles.end()-1));
					handles.erase(handles.end()-1);
				}
			}
		}
	}
	// Shift the data in the data buffer
	for(unsigned i=0;i<alldata.size();i++)
	{
		// Erase old data
		alldata[i].erase(alldata[i].begin());
		// Store new data
		alldata[i].push_back(linedata[i]);
	}
	/*if(PreciseTimer::QueryTimer()-lastplot>refresh)
	{
		Plot();
		dscopes->Render();
		dscopes->repaint();
		lastplot = PreciseTimer::QueryTimer();
	}*/
}
void MainWindow::displayError(QAbstractSocket::SocketError)
{
	printf("Socket error\n");
}


void MainWindow::SetRefreshRate(int i)
{
	killTimer(timer);
	timer=startTimer(1000/i);
	refresh=1.0/(double)i;	
}
void MainWindow::SetAfterGlow(int state)
{
	dscopes->SetAlpha(state==Qt::Checked);
}


void MainWindow::applyClicked()
{
	sd.scopedefinition.clear();
	//int rv = ParseLayoutString(uileDisplayFormat->text().toStdString(),sd);
	int rv = ParseLayoutString(uiteDisplayFormat->toPlainText().toStdString(),sd);
	/*if(rv!=0)
	{
		printf("Error in layout string\n");
		QMessageBox::information(this, "Invalid display format string", 
"Display string format: [numtraces]<scopedef>[<scopedef>]...\n\
\n\
  numtraces: Optionally specifies the number of traces. The layout is applied only if the effective number of traces matches numtraces\n\
  scopedef:  String defininig the traces to display on the scope, title, axis ranges and colors\n\
\n\
scopedef format: <<title>;<traces>[;xscale=<xs>][;yscale=<auto|min;max>][color=<colors>]>\n\
\n\
  title:  Title of the scope\n\
  traces: Space separated list of traces to display on this scope\n\
  xscale: Horizontal scaling factor. Positive number: zoom in. Negative number: zoom out\n\
  yscale: Vertical scale: 'min max' or 'auto'\n\
  color:  Space separated list of colors for each trace. Colors are hexadecimal numbers with format 0xRRGGBBXX for RGB color RR,GG,BB.\n\
\n\
Example layout string: <Scope 1;1 2 3;xscale=1;yscale=auto;color=0xff000000 0x00ff0000 0x0000ff00><Scope 2;4;yscale=-100 100>\n");
	}
	printf("sd scopedefinition size: %d\n",sd.scopedefinition.size());*/
	
	// Erase all the scopes
	while(handles.size())
	{
		dscopes->RemoveScope(*(handles.end()-1));
		handles.erase(handles.end()-1);
	}	
	// Hardcore init of the number of scopes
	if(sd.scopedefinition.size()!=0)
	{
		
		handles=dscopes->AddScopes(sd.scopedefinition.size());
		for(unsigned i=0;i<sd.scopedefinition.size();i++)
		{
			dscopes->operator[](handles[i])->SetTitle(sd.scopedefinition[i].title);
			if(sd.scopedefinition[i].yauto)
				dscopes->operator[](handles[i])->SetVAuto();
			else
				dscopes->operator[](handles[i])->SetVRange(sd.scopedefinition[i].yscale[0],sd.scopedefinition[i].yscale[1]);
			dscopes->operator[](handles[i])->HZoom(sd.scopedefinition[i].xscale);
		}
		vector<int> v(1,0);
		for(unsigned i=0;i<sd.scopedefinition.size();i++)
			dscopes->operator[](handles[i])->Plot(v);
	}
	else	// Variable number of scopes: adjust to the number of data last received
	{
		handles = dscopes->AddScopes(alldata.size());
	}
	dscopes->Render();
	dscopes->repaint();
	//mutex.unlock();	
}
void MainWindow::applyBinary()
{
	// DX3 reader: DX3;cc-s-s-s-s-s-s
	
	// Test the binary string.
	string binaryformat = uiteBinaryFormat->toPlainText().trimmed().toStdString();
	if(binaryformat.size()==0)
	{
		binary=false;
	}
	else
	{
		binary=true;
		fp = FrameParser2(binaryformat);
		if(!fp.IsValid())
			QMessageBox::critical(this, "Invalid binary format", "Binary format specification is: \"<header>;<[-]c|s|S|i|I><[-]c|s|S|i|I>...\"");
	}
}

/*void MainWindow::stdioRead()
{
	printf("stdioRead()\n");
}
*/
/*
Benchmark image ARGB:
	Time of Plot: 0.026112
	Time of repaint: 0.035359
*/

void MainWindow::testClicked()
{
	close();
	return;
	
	printf("Benchmark\n");
	double t1,t2;
	int i;	

	dscopes->pt=0;
	dscopes->rt=0;

	// Init the scopes
	if(handles.size()==0)
		handles = dscopes->AddScopes(10);
	alldata.resize(10,vector<int>(10000,0));
	for(i=0;i<10000;i++)
	{
		for(int j=0;j<10;j++)
		{
			alldata[j][i] = 1000*sin((double)(i/10+j))*(j+1);
		}
	}


	t1=PreciseTimer::QueryTimer();
	for(i=0;i<200;i++)
		Plot();
	t2=PreciseTimer::QueryTimer();
	printf("Time of Plot: %lf ms\n",(t2-t1)/(double)i*1000.0);
	t1=PreciseTimer::QueryTimer();
	for(i=0;i<200;i++)
		dscopes->Render();
	t2=PreciseTimer::QueryTimer();
	printf("Time of Render: %lf ms\n",(t2-t1)/(double)i*1000.0);
	t1=PreciseTimer::QueryTimer();
	for(i=0;i<200;i++)
		dscopes->repaint();
	t2=PreciseTimer::QueryTimer();
	printf("Time of repaint: %lf ms\n",(t2-t1)/(double)i*1000.0);
	//t1=PreciseTimer::QueryTimer();
	//while(PreciseTimer::QueryTimer()-t1<1.0);
	//printf("render time: %lf paint time: %lf. render+paint: %lf ms\n",dscopes->rt*1000.0,dscopes->pt*1000.0,(dscopes->rt+dscopes->pt)*1000.0);
	QMessageBox::information(this, "end","end");
	
	
}
/*
DEBUG:
img size: 3145728
QImage::Format_RGB32. Time of Plot: 20.749023 ms
QImage::Format_RGB32. Time of clear: 4.681715 ms
QImage::Format_RGB32. Time of memset clear: 2.716775 ms
img size: 3145728
QImage::Format_ARGB32_Premultiplied. Time of Plot: 21.121212 ms
QImage::Format_ARGB32_Premultiplied. Time of clear: 4.708499 ms
QImage::Format_ARGB32_Premultiplied. Time of memset clear: 2.730353 ms
img size: 3145728
QImage::Format_ARGB32. Time of Plot: 31.055089 ms
QImage::Format_ARGB32. Time of clear: 4.755296 ms
QImage::Format_ARGB32. Time of memset clear: 2.747260 ms
img size: 1572864
QImage::Format_RGB16. Time of Plot: 17.841656 ms
QImage::Format_RGB16. Time of clear: 2.360811 ms
QImage::Format_RGB16. Time of memset clear: 1.402953 ms
img size: 2359296
QImage::Format_RGB666. Time of Plot: 20.018832 ms
QImage::Format_RGB666. Time of clear: 3.692228 ms
QImage::Format_RGB666. Time of memset clear: 2.117514 ms
img size: 2359296
QImage::Format_RGB888. Time of Plot: 19.868314 ms
QImage::Format_RGB888. Time of clear: 3.616176 ms
QImage::Format_RGB888. Time of memset clear: 2.058403 ms

RELEASE:
Benchmark of single DScopeQT with various image formats
img size: 3145728
QImage::Format_RGB32. Time of Plot: 10.510114 ms
QImage::Format_RGB32. Time of clear: 2.665050 ms
QImage::Format_RGB32. Time of memset clear: 2.702188 ms
img size: 3145728
QImage::Format_ARGB32_Premultiplied. Time of Plot: 10.449322 ms
QImage::Format_ARGB32_Premultiplied. Time of clear: 2.668212 ms
QImage::Format_ARGB32_Premultiplied. Time of memset clear: 2.686137 ms
img size: 3145728
QImage::Format_ARGB32. Time of Plot: 21.323910 ms
QImage::Format_ARGB32. Time of clear: 2.663572 ms
QImage::Format_ARGB32. Time of memset clear: 2.684926 ms
img size: 1572864
QImage::Format_RGB16. Time of Plot: 8.796347 ms
QImage::Format_RGB16. Time of clear: 1.367266 ms
QImage::Format_RGB16. Time of memset clear: 1.391469 ms
img size: 2359296
QImage::Format_RGB666. Time of Plot: 10.878533 ms
QImage::Format_RGB666. Time of clear: 2.881362 ms
QImage::Format_RGB666. Time of memset clear: 2.030368 ms
img size: 2359296
QImage::Format_RGB888. Time of Plot: 10.486872 ms
QImage::Format_RGB888. Time of clear: 2.813615 ms
QImage::Format_RGB888. Time of memset clear: 2.031343 ms
Time of memset: 2.684919 ms

DEBUG:
	Time RGB32, 1024x768
	
	Standard plot					20.9
	Plot w/o string, w/o clear:		10.9 ms
	Plot w/o line+setpen:			9.8ms
	Plot w/o line:					10.3ms
	Plot w/o setpen:				19.8ms
	Time for painter.fillRect:		5ms
	Time for string:				5ms
	
RELEASE
	Time RGB32, 1024x768
	Standard plot					10.6
	Plot w/o string, w/o clear:		7.9 ms
	
SDLScopeTest -O2:
	1024x768x16
		5.1ms Plot
		5.2ms SDL_UpdateRect
	1024x768x32
		9.1ms Plot
		4.9ms SDL_UpdateRect
	-00:
	1024x768x16
		4.6ms Plot
		5.1ms SDL_UpdateRect
	1024x768x32
		6.9ms Plot
		4.9ms SDL_UpdateRect
		
		
Optimization of string:
	RELEASE qt:			10.7	10.8	10.9	10.7
	RELEASE direct:		8.8		8.7		8.9		8.9
	debug qt:			20.9	21.0	21.5	21.1
	debug direct:		19.8	19.6	18.3	18.8

Optimization of vline+hline (implied opt string):
	RELEASE qt:			8.8		8.9		8.9		8.9
	RELEASE	direct:		9.5		9.5		9.5		9.4
*/

/*void MainWindow::testClicked()
{
	printf("Benchmark of single DScopeQT with various image formats\n");
	double t1,t2;
	int i;	

	dscopes->pt=0;
	dscopes->rt=0;

	// Init the scopes
	if(handles.size()==0)
		handles = dscopes->AddScopes(10);
	alldata.resize(10,vector<int>(10000,0));
	for(i=0;i<10000;i++)
	{
		for(int j=0;j<10;j++)
		{
			alldata[j][i] = 1000*sin((double)(i/10+j))*(j+1);
		}
	}

	QImage::Format f[]={	
					QImage::Format_RGB32,
					QImage::Format_ARGB32_Premultiplied,
					QImage::Format_ARGB32,
					QImage::Format_RGB16,
					QImage::Format_RGB666,
					QImage::Format_RGB888};
	char *str[] = {
		"QImage::Format_RGB32",
		"QImage::Format_ARGB32_Premultiplied",
		"QImage::Format_ARGB32",
		"QImage::Format_RGB16",
		"QImage::Format_RGB666",
		"QImage::Format_RGB888",
		"QPixmap"
	};
	int w=1024;
	int h=768;
	DScopeQT *scope;
	QImage *img=0;
	for(int ff = 0;ff<1;ff++)
	{
		if(img) {
			delete img; img=0;
		}
		
		if(ff<6)
			img = new QImage(w,h,f[ff]);
		//else
			//img = new QPixmap(w,h);
			
		scope=new DScopeQT(img,0,0,w,h,false);
		
		
		printf("img size: %d\n",img->numBytes());
		
		t1=PreciseTimer::QueryTimer();
		for(i=0;i<200;i++)
			scope->Plot(alldata[0]);
		t2=PreciseTimer::QueryTimer();
		printf("%s. Time of Plot: %lf ms\n",str[ff],(t2-t1)/(double)i*1000.0);
		
		t1=PreciseTimer::QueryTimer();
		QPainter painter(img);
		//printf("renderhint: %X\n",painter.renderHints());
		painter.setClipRect(0,0,w,h);
		for(i=0;i<200;i++)
		{
			painter.fillRect(0,0,w,h,Qt::black);	
		}
		t2=PreciseTimer::QueryTimer();
		printf("%s. Time of clear: %lf ms\n",str[ff],(t2-t1)/(double)i*1000.0);
		
		
		
		t1=PreciseTimer::QueryTimer();
		for(i=0;i<200;i++)
		{
			memset(img->bits(),0,img->numBytes());
		}
		t2=PreciseTimer::QueryTimer();
		printf("%s. Time of memset clear: %lf ms\n",str[ff],(t2-t1)/(double)i*1000.0);


		delete scope;
	}
	char *b=new char[w*h*4];
	t1=PreciseTimer::QueryTimer();
	for(i=0;i<200;i++)
	{
		memset(b,0,w*h*4);
	}
	t2=PreciseTimer::QueryTimer();
	printf("Time of memset: %lf ms\n",(t2-t1)/(double)i*1000.0);
	delete[] b; 	
	
}




*/

void MainWindow::testDashel()
{/*
	std::map<int, std::pair<std::string, std::string> > ports = SerialPortEnumerator::getPorts();

	for(std::map<int, std::pair<std::string, std::string> >::iterator it = ports.begin(); it != ports.end(); ++it)
	{
		std::cout << "(" << it->first << ") " << it->second.first << " [" << it->second.second << "]" << std::endl;
	}*/
	
	// Test QSerial stuff
	QList<QextPortInfo> ports = QextSerialEnumerator::getPorts();
	printf("List of ports:\n");
	for (int i = 0; i < ports.size(); i++) {
		printf("port name: %s\n", ports.at(i).portName.toLocal8Bit().constData());
		printf("friendly name: %s\n", ports.at(i).friendName.toLocal8Bit().constData());
		printf("physical name: %s\n", ports.at(i).physName.toLocal8Bit().constData());
		printf("enumerator name: %s\n", ports.at(i).enumName.toLocal8Bit().constData());
		printf("===================================\n\n");
	}
	// Does not correclty set the port parameters!
	/*PortSettings ps;
	ps.BaudRate=BAUD19200;
	ps.DataBits=DATA_8;
	ps.FlowControl=FLOW_OFF;
	ps.Parity=PAR_NONE;
	ps.StopBits=STOP_1;
	ps.Timeout_Millisec=500;
	
	port = new QextSerialPort("COM4",ps,QextSerialBase::EventDriven);
	port->open(QIODevice::ReadWrite);
	connect(port,SIGNAL(readyRead()),this,SLOT(serialRead()));*/
	//listener->connect(port, SIGNAL(readyRead()), listener, SLOT(serialRead()));
	//listener->connect(port, SIGNAL(bytesWritten(qint64)), listener, SLOT(reportWritten(qint64)));
	//listener->connect(port, SIGNAL(aboutToClose()), listener, SLOT(reportClose()));
	//listener->connect(port, SIGNAL(dsrChanged(bool)), listener, SLOT(reportDsr(bool)));	
	
	
	// XML test
	QString string;
	QXmlStreamWriter xml(&string);
	xml.writeStartDocument();
	xml.writeStartElement("Configuration") ;
	xml.writeAttribute("DisplayFormat",uiteDisplayFormat->toPlainText());
	xml.writeAttribute("BinaryFormat",uiteBinaryFormat->toPlainText());
	xml.writeAttribute("InputDevice",uileHostPort->text());
	xml.writeEndElement();
	xml.writeEndDocument();
	
	printf("string: '%s'\n",string.toStdString().c_str());
	
	
	
	
}
void MainWindow::tcpRead()
{
	printf("tcpRead bytes available: %d\n",tcpSocket->bytesAvailable());
	if(binary)
	{
		QByteArray ba=tcpSocket->readAll();
		BinaryChunckDecodeRead(ba);	
	}
	else
	{
		QTextStream in(tcpSocket);
		TextChunckDecodeRead(in);
	}
}
void MainWindow::serialRead()
{
	if(binary)
	{
		QByteArray ba=port->readAll();
		BinaryChunckDecodeRead(ba);	
	}
	else
	{
		QTextStream in(port);
		TextChunckDecodeRead(in);
	}
}
void MainWindow::TextChunckDecodeRead(QTextStream &in)
{
	int numsample=0;
	double t2,t1;
	t1 = PreciseTimer::QueryTimer();
	while(1)
	{
		QString qstr = in.readLine();
		if(qstr.isNull())
			break;
		QRegExp rx(QString("[ ;,\t]"));			// Match any separator
		QStringList sl = qstr.split(rx);		// Split on separator
		vector<int> linedata;
		for(int i=0;i<sl.size();i++)			// Convert string list to int vector 
		{
			bool ok;
			int v = sl[i].toInt(&ok);
			if( ok )
			{
				linedata.push_back(v);
			}
		}		
		receivedData(linedata);
	}
}
void MainWindow::BinaryChunckDecodeRead(QByteArray &ba)
{
	vector<vector<int> > v = fp.Parser(ba.data(),ba.size());
	if(v.size())
	{
		for(int i=0;i<v.size();i++)
		{
			receivedData(v[i]);
		}
	}
}
void MainWindow::dsrChanged(bool s)
{
	printf("dsrChanged status: %d\n",s);
}
void MainWindow::saveConfiguration()
{
	QString fileName = QFileDialog::getSaveFileName(this, "Save Configuration",QString(),"Configuration (*.xml)");
	if(!fileName.isNull())
	{
		QFile file(fileName);
		if (file.open(QIODevice::WriteOnly | QIODevice::Text | QIODevice::Truncate))
		{
			QXmlStreamWriter xml(&file);
			xml.writeStartDocument();
			xml.writeStartElement("Configuration") ;
			xml.writeAttribute("DisplayFormat",uiteDisplayFormat->toPlainText());
			xml.writeAttribute("BinaryFormat",uiteBinaryFormat->toPlainText());
			xml.writeAttribute("InputDevice",uileHostPort->text());
			xml.writeEndElement();
			xml.writeEndDocument();
			file.close();
		}
		else
		{
			QMessageBox::critical(this, "Save configuration error", "Cannot write to file");
		}
	}
}
void MainWindow::loadConfigurationAction()
{
	QString fileName = QFileDialog::getOpenFileName(this, "Load Configuration",QString(),"Configuration (*.xml)");
	if(!fileName.isNull())
		loadConfiguration(fileName);
}
void MainWindow::loadConfiguration(QString fileName)
{
	QFile file(fileName);
	if (file.open(QIODevice::ReadOnly | QIODevice::Text))
	{
		QXmlStreamReader xml(&file);
		while (!xml.atEnd())
		{	
			QXmlStreamReader::TokenType type = xml.readNext();
			//printf("readNext type: %d\n",type);
			if(type==QXmlStreamReader::StartElement)
			{
				//printf("name: %s\n",xml.name().toString().toStdString().c_str());
				//printf("namespaceUri: %s\n",xml.namespaceUri().toString().toStdString().c_str());
				if(xml.name().toString().compare("Configuration")==0)
				{
					QXmlStreamAttributes at = xml.attributes();
					QString df = at.value("DisplayFormat").toString();
					QString bf = at.value("BinaryFormat").toString();
					QString id = at.value("InputDevice").toString();
					if(!df.isNull())
						uiteDisplayFormat->setPlainText(df);
					if(!bf.isNull())
						uiteBinaryFormat->setPlainText(bf);
					if(!id.isNull())
						uileHostPort->setText(id);
					applyClicked();
					applyBinary();
				}
			}
		}
		file.close();
	}
	else
	{
		QMessageBox::critical(this, "Load configuration error", "Cannot read from file");
	}	
}
void MainWindow::actionExit()
{
	close();
}

void MainWindow::actionAbout()
{
	QMessageBox::about(this, "About",
	"<p><b>DScope</b> - QT Version</p>\n"
	"<p>Version 1.00</p>"
	"<p>(c) 2007,2008 Daniel Roggen</p>");

}
void MainWindow::actionListPorts()
{
	PortWindowImpl dialog(this);
	int ret = dialog.exec();
	if (ret == QDialog::Rejected)
		return;
}
void MainWindow::actionHowto()
{
	HelpWindowImpl dialog(this);
	dialog.exec();	
}
	
	
	
	
